/*
Copyright © 2022 Volodymyr Patuta me@vpatuta.xyz

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/
package cmd

import (
	"os"
	"time"

	"github.com/jedib0t/go-pretty/v6/table"
	"github.com/jedib0t/go-pretty/v6/text"
	"github.com/spf13/cobra"
	"golang.org/x/sys/unix"
)

// lsCmd represents the ls command
var lsCmd = &cobra.Command{
	Use:   "ls",
	Short: "List todos",
	Long: `List todos. By default, lists uncompleted todos.

If given '-c' flag, lists completed todos.
Otherwise, you can list all todos with '-a'.

You can list showing all fields with '-v'.`,
	Run: func(cmd *cobra.Command, args []string) {
		var todos []Todo
		all, err := cmd.Flags().GetBool("all")
		errPanic(err, "")
		comp, err := cmd.Flags().GetBool("completed")
		errPanic(err, "")
		verbose, err := cmd.Flags().GetBool("verbose")
		errPanic(err, "")
		if all {
			todos = getAllTodos()
		} else if comp {
			todos = getDoneTodos()
		} else {
			todos = getUnDoneTodos()
		}
		ws, err := unix.IoctlGetWinsize(int(os.Stdin.Fd()), unix.TIOCGWINSZ)
		errPanic(err, "failed to determine terminal width")
		cols := int(ws.Col)
		t := table.NewWriter()
		t.SetOutputMirror(os.Stdout)
		if verbose {
			t.AppendHeader(table.Row{"#", "Description", "Due date", "Completed", "Created at"})
		} else {
			t.AppendHeader(table.Row{"#", "Description", "Due date", "Completed"})
		}
		for _, todo := range todos {
			date := todo.DueDate.Format("02/01/2006 15:04")
			d := time.Time{}
			if todo.DueDate == d {
				date = ""
			}
			desc := todo.Desc
			if len(desc) > (cols - 20) {
				desc = insertNth(desc, cols-20)
			}
			comp := '✗'
			if todo.IsDone {
				comp = '✓'
			}
			if verbose {
				t.AppendRow([]interface{}{todo.ID, desc, date, string(comp), todo.CreatedAt.Format("02/01/2006 15:04")})
			} else {
				t.AppendRow([]interface{}{todo.ID, desc, date, string(comp)})
			}
		}
		t.SetStyle(table.StyleLight)
		t.SetAllowedRowLength(cols)
		t.SetColumnConfigs([]table.ColumnConfig{
			{Number: 4, Align: text.AlignCenter},
		})
		t.Render()
	},
}

// insertNth inserts new line on each nth position.
func insertNth(s string, n int) string {
	for i := n - 1; i < len(s); i += n {
		s = s[:i] + "\n" + s[i:]
	}

	return s
}

func init() {
	rootCmd.AddCommand(lsCmd)
	lsCmd.Flags().BoolP("all", "a", false, "show all todos")
	lsCmd.Flags().BoolP("completed", "c", false, "show all completed todos")
	lsCmd.Flags().BoolP("verbose", "v", false, "show all fields of todos")
	lsCmd.MarkFlagsMutuallyExclusive("all", "completed")
}
